/* Copyright (C) 2001 W.P. van Paassen - peter@paassen.tmfweb.nl, E.C. Brummel
   

   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

//***************************************************************************
//***                       CLASS CELLPANELFACTORY                        ***
//***************************************************************************

import java.awt.*;
import java.util.*;

public class CellPanelFactory{

    private SpeelVeldPanel parent;
    private SingleCellPanel[] singleCellPanels;
    private int aantalFreeCells;
    private int aantalHomeCells;
    private int aantalStackCells;
    private Vector[] stackCellKaarten;

    public CellPanelFactory(SpeelVeldPanel parent, SingleCellPanel[] singleCellPanels,
                                    int aantalFreeCells, int aantalHomeCells,
                                        int aantalStackCells, Vector[] stackCellKaarten){

        this.parent = parent;
        this.singleCellPanels = singleCellPanels;
        this.aantalFreeCells = aantalFreeCells;
        this.aantalHomeCells = aantalHomeCells;
        this.aantalStackCells = aantalStackCells;
        this.stackCellKaarten = stackCellKaarten;
    }

    //METHOD createFreeCellPanel
    public LightWeightPanel createFreeCellPanel(){
        int index = aantalStackCells;
        LightWeightPanel freeCellPanel = new LightWeightPanel();
        freeCellPanel.setLayout(new GridLayout(1, aantalFreeCells, 10, 0));
        while(index < aantalStackCells + aantalFreeCells){
            SingleFreeCellPanel sfcp = new SingleFreeCellPanel(index);
            singleCellPanels[index] = sfcp;
            freeCellPanel.add(sfcp);
            sfcp.addPropertyChangeListener(parent);
            index++;
        }
        return freeCellPanel;
    }

    //METHOD createHeadPanel
    public HeadPanel createHeadPanel(){
        return new HeadPanel();
    }

    //METHOD createHomeCellPanel
    public LightWeightPanel createHomeCellPanel(){
        int index = 12;
        LightWeightPanel homeCellPanel = new LightWeightPanel();
        homeCellPanel.setLayout(new GridLayout(1, aantalHomeCells, 10, 0));
        int aantalCells = aantalFreeCells + aantalHomeCells + aantalStackCells;
        while (index < aantalCells){
            SingleHomeCellPanel shcp = new SingleHomeCellPanel(index);
            singleCellPanels[index] = shcp;
            homeCellPanel.add(shcp);
            shcp.addPropertyChangeListener(parent);
            index++;
        }
        return homeCellPanel;
    }

    //METHOD createStackCellPanel
    public LightWeightPanel createStackCellPanel(){
        int index = 0;
        LightWeightPanel stackCellPanel = new LightWeightPanel();
        stackCellPanel.setLayout(new GridLayout(1, aantalStackCells, 6, 0));
        while (index < aantalStackCells){
            Vector kaarten = stackCellKaarten[index];
            SingleStackCellPanel sscp = new SingleStackCellPanel(index);
            singleCellPanels[index] = sscp;
            stackCellPanel.add(sscp);
            sscp.drawCell(kaarten);
            sscp.addPropertyChangeListener(parent);
            index++;
        }
        return stackCellPanel;
    }
}


