/* Copyright (C) 2001 W.P. van Paassen - peter@paassen.tmfweb.nl, E.C. Brummel
   

   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

//***************************************************************************
//***                       CLASS SINGLEFREECELLPANEL                     ***
//***************************************************************************

import java.awt.*;
import java.awt.event.*;
import java.beans.*;

class SingleFreeCellPanel extends SingleCellPanel{
    
    private int ID;
    private boolean selected;
    private Image card;
    private Image background;
    private PropertyChangeSupport pcs = new PropertyChangeSupport(this);
    
    //CONSTRUCTOR
    public SingleFreeCellPanel(int ID){
        this.ID = ID;
        setSize(getPreferredSize());
    }
    
    //METHOD setBackgroundImage
    public void setBackgroundImage(Image image){
        background = image;
    }
    
    //METHOD getBackgroundImage
    public Image getBackgroundImage(){
        return background;
    }

    //METHOD paint
    public void paint(Graphics g){
        g.fill3DRect(0, 0, 75, 100, false);
        if (card != null){
            g.drawImage(card, 2, 2, this);
            if (selected){
                g.setXORMode(new Color(255, 79, 196));
                g.fillRect(3, 3, 69, 94);
            }
        }
        else
            g.drawImage(background, 2, 2, this);
    }

    //METHOD drawCell
    public void drawCell(Object object){
        Kaart kaart = (Kaart)object;
        if (kaart != null) card = GameFrame.cards[kaart.getCardNumber()];
        else card = null;
        repaint();
    }

    //METHOD highlightCard
    public void highlightCard(byte kaartRang){
        if (!selected){
            Graphics g = getGraphics();
            g.setXORMode(new Color(255, 79, 196));
            g.fillRect(3, 3, 69, 94);
        }
    }

    //METHOD select
    public boolean select(){
        if (card != null){
            selected = true;
            Graphics g = getGraphics();
            g.setXORMode(new Color(255, 79, 196));
            g.fillRect(3, 3, 69, 94);
        }
        return selected;
    }
    
    //Method deselect
    public boolean deselect(){
        if (selected){
            Graphics g = getGraphics();
            g.setXORMode(new Color(255, 79, 196));
            g.fillRect(3, 3, 69, 94);
        }
        selected = false;
        return selected;
    }
    
    //METHOD mousePressed
    public void mousePressed(MouseEvent evt){
            pcs.firePropertyChange("clicked", null, new Integer(ID));
    }
          
    //METHOD mouseMoved
    public void mouseMoved(MouseEvent evt){
        pcs.firePropertyChange("Kupke",new Integer(ID),null);
    }
    
    //METHOD getPreferredSize
    public Dimension getPreferredSize(){ return new Dimension(75, 100); }
    
	//METHOD addPropertyChangeListener
	public void addPropertyChangeListener(PropertyChangeListener l){
		pcs.addPropertyChangeListener(l);
	}
	
	//METHOD removePropertyChangeListener
	public void removePropertyChangeListener(PropertyChangeListener l){
		pcs.removePropertyChangeListener(l);
	}
}
