/* Copyright (C) 2001 W.P. van Paassen - peter@paassen.tmfweb.nl, E.C. Brummel
   

   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

//***************************************************************************
//***                       CLASS SPEELVELD                               ***
//***************************************************************************

import java.io.*;
import java.util.*;
import java.beans.*;

public class SpeelVeld
{
  
    private Cell[] cells;
    private short zetIndex;
    private byte aantalKaarten;
      
    //DUMMY CONSTRUCTOR
    public SpeelVeld(){
    	//dummy voor deepClone()
    }
    
    //CONSTRUCTOR
    public SpeelVeld(Kaart[] deKaartenRij, int aantalFreeCells, int aantalHomeCells, int aantalStackCells){
    	aantalKaarten = (byte) deKaartenRij.length;
    	creeerCells(deKaartenRij, aantalFreeCells, aantalHomeCells, aantalStackCells);
    }
   
    //METHOD creeerCells
    public void creeerCells(Kaart[] deKaartenRij, int aantalFreeCells, int aantalHomeCells, int aantalStackCells){
   	int aantalcells = aantalFreeCells + aantalHomeCells + aantalStackCells;
        cells = new Cell[aantalcells];
        int index = 0;
        
        //VUL DE CELLS MET STACKCELLS
        int stackcellindex = 0;
        int kaartindex = 0;
       	while(index != aantalStackCells){
         	cells[index] = new Stack_Cell();
         	index++;
        }
        //VUL DE STACKCELLS MET KAARTEN   
        while (kaartindex != aantalKaarten){
         	cells[stackcellindex].setTopKaart(deKaartenRij[kaartindex]);
         	stackcellindex++;
         	stackcellindex %= aantalStackCells;
         	kaartindex++;
        }
        //VUL DE CELLS MET FREECELLS
        while (index < aantalStackCells + aantalFreeCells){
         	cells[index] = new Free_Cell();  
         	index++; 
        }
        //VUL DE CELLS MET HOMECELLS
        while (index < aantalcells){
         	cells[index] = new Home_Cell();  
         	index++; 
        }
    }
    
    //=================================================================================
    //                              METHODEN VOOR GUI
    //=================================================================================
    
    //METHOD createNewCardDeck
    public void createNewCardDeck(Kaart[] kaartenRij){
        //ALLES UIT DE CELLEN VERWIJDEREN
        for(int i = 0; i < cells.length; i++){
            cells[i].reset();
        }
        //EN DE STACKCELLS VULLEN MET EEN NIEWE KAARTSTAPEL
        int stackcellindex = 0;
    	int kaartindex = 0;
    	aantalKaarten = (byte) kaartenRij.length;
        while (kaartindex != aantalKaarten){
         	cells[stackcellindex].setTopKaart(kaartenRij[kaartindex]);
         	stackcellindex++;
         	stackcellindex %= 8;
         	kaartindex++;
        }
    }
    
    //METHOD getKaartenUitStackCell
    public Vector getKaartenUitStackCell(int cellNr){
        Stack_Cell sc = (Stack_Cell) cells[cellNr];
        Vector kaarten = sc.getKaarten();
        return kaarten;
    }

    //METHOD getCell
    public Cell getCell(int cellNummer){
        return cells[cellNummer];
    }
    
    //METHOD changeZetIndex
    public void changeZetIndex(int from, int to){
        for (short index = 0; index < 228; index++){
            if (Spel.zetten[index][0] == from && Spel.zetten[index][1] == to){
                zetIndex = index;
                return;
            }
        }
    }
    
    //METHOD geefCellInhoud
    public Object geefCellInhoud(int index){
        Cell cell = cells[index];
       	if (cell instanceof Stack_Cell){
      	    Stack_Cell stack_cell = (Stack_Cell)cell;
      	    return stack_cell.getKaarten();
      	}
        return cell.getTopKaart(); 
    }

    //METHOD geefCellInhoudFrom
    public ObjectContainer geefCellInhoudFrom(){
        return new ObjectContainer (geefCellInhoud(Spel.zetten[zetIndex][0]),
        Spel.zetten[zetIndex][0]);
    }

    //METHOD geefCellInhoudTo
    public ObjectContainer geefCellInhoudTo(){
        return new ObjectContainer(geefCellInhoud(Spel.zetten[zetIndex][1]),
        Spel.zetten[zetIndex][1]);
    }

    //METHOD geefLocatiesKaart
    public Vector geefLocatiesKaart(byte kaartRang){
        Vector v = new Vector();
        for (byte i = 0; i < cells.length; i++){
            Cell cell = cells[i];
            if (cell instanceof Stack_Cell){
                if (gezochteKaartInStackCell(cell, kaartRang))
                    v.addElement(new Byte(i));
            }
            else{
                Kaart kaart = cell.getTopKaart();
                if (kaart != null){
                    if (kaart.getRang() == kaartRang)
                        v.addElement(new Byte(i));
                }
            }
        }
        return v;
    }

    //METHOD gezochteKaartInStackCell
    public boolean gezochteKaartInStackCell(Cell cell, byte kaartRang){
        Stack_Cell stackCell = (Stack_Cell)cell;
        Vector kaarten = stackCell.getKaarten();
        int aantal = stackCell.getAantalKaarten();

        for (int index = 0; index < aantal; index++){
            Kaart kaart = (Kaart)kaarten.elementAt(index);
            if (kaart.getRang() == kaartRang) return true;                
        }
        return false;
    }

    //METHOD geefOverbodigeKaart
    public int[] geefOverbodigeKaart(){
        byte laagsteRangZwart = 13;  
        byte laagsteRangRood = 13;
        for (int i = 12; i < 16; i++){
            Cell cell = cells[i];
            Kaart kaart = cell.getTopKaart();
            if (kaart != null){
                if (kaart.getKleur().equals("zwart")){
                    if (kaart.getRang() < laagsteRangZwart)
                        laagsteRangZwart = kaart.getRang();
                }
                else{
                    if (kaart.getRang() < laagsteRangRood)
                        laagsteRangRood = kaart.getRang();
                }
            }
        }
        for (int i = 0; i < 12; i++){
            Cell cell = cells[i];
            Kaart kaart = cell.getTopKaart();
            if (kaart != null){
                int to = waarMoetKaartHeen(kaart);
                if (kaart.getKleur().equals("zwart")){
                    if (zijnBeideKaartenVanKleurAanwezig("rood") ||
                            kaart.getRang() == Kaart.Aas || kaart.getRang() == 2){
                        if (kaart.getRang() - laagsteRangRood <= 1 || kaart.getRang() == Kaart.Aas){
                            if (to != -1){
                                int[] fromToRij = new int[2];
                                fromToRij[0] = i;
                                fromToRij[1] = to;
                                return fromToRij;
                            }
                        }
                    }        
                }
                else{
                    if (zijnBeideKaartenVanKleurAanwezig("zwart") ||
                            kaart.getRang() == Kaart.Aas || kaart.getRang() == 2){
                        if (kaart.getRang() - laagsteRangZwart <= 1 || kaart.getRang() == Kaart.Aas){
                            if (to != -1){
                                int[] fromToRij = new int[2];
                                fromToRij[0] = i;
                                fromToRij[1] = to;
                                return fromToRij;
                            }
                        }
                    }        
                }
            }
        }
        return null;
    }

    //METHOD zijnBeideKaartenVanKleurAanwezig
    public boolean zijnBeideKaartenVanKleurAanwezig(String kleur){
        int aantal = 0;
        for (int i = 12; i < 16; i++){
            Cell cell = cells[i];
            Kaart kaart = cell.getTopKaart();
            if (kaart != null){
                    if (kaart.getKleur().equals(kleur)) aantal++;
            }
        }
        return aantal == 2;
    }

    //METHOD waarMoetKaartHeen
    public int waarMoetKaartHeen(Kaart kaart){
        for (int i = 12; i < 16; i++){
            Cell cell = cells[i];
            Kaart homeKaart = cell.getTopKaart();
            if (homeKaart != null){
                    if (kaart.getKaartSoort() == homeKaart.getKaartSoort()
                           && kaart.isEenHogerInRangDan(homeKaart))
                                    return i;
            }
            else if (kaart.getRang() == Kaart.Aas) return i;
        }
        return -1; //Deze kaart kan niet opgeruimd worden!
    }
    
    //METHOD getAantalLegeFreeCells
    public int getAantalLegeFreeCells(){
    	int aantal = 0;
    	for(int i = 8; i <= 11; i++){
    		if(!cells[i].bevatKaart()) aantal++;
    	}
    	return aantal;
    }
    
    //METHOD getAantalLegeStackCells
    public int getAantalLegeStackCells(){
    	int aantal = 0;
    	for(int i = 0; i <= 7; i++){
    		if(!cells[i].bevatKaart()) aantal++;
    	}
    	return aantal;	
    }
	
    //METHOD inFinalState
    public boolean inFinalState(){
         return cells[12].getAantalKaarten() + cells[13].getAantalKaarten()
         + cells[14].getAantalKaarten() + cells[15].getAantalKaarten()
         == aantalKaarten;
    }
	
    //METHOD changeState
    public void changeState(){
         short from = Spel.zetten[zetIndex][0];
         short to = Spel.zetten[zetIndex][1];
      
         cells[to].setTopKaart(cells[from].getTopKaart());
         cells[from].removeTopKaart();
    }
}    
