/* Copyright (C) 2001 W.P. van Paassen - peter@paassen.tmfweb.nl, E.C. Brummel
   

   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

//***************************************************************************
//***                       CLASS SPEELVELDPANEL                          ***
//***************************************************************************

import java.awt.*;
import java.beans.*;
import java.util.*;
import java.awt.event.*;

class SpeelVeldPanel extends Panel implements PropertyChangeListener, ActionListener {
    
    private HeadPanel headPanel;
    private Panel buttonPanel;
    private Panel menuPanel;
    private TextField textField;
    private SingleCellPanel[] singleCellPanels;
    private GridBagConstraints gbc;
	private AboutDialog about;
	private HelpDialog help;
	private KolomDialog kolom;
	
	private boolean selected = false;
    private int fromId;
	private int toId;
    private PropertyChangeSupport pcs = new PropertyChangeSupport(this);
    private VetoableChangeSupport vcs = new VetoableChangeSupport(this);
    
    //CONSTRUCTOR
    public SpeelVeldPanel(int aantalFreeCells, int aantalHomeCells,
                            int aantalStackCells, Vector[] stackCellKaarten){

        setBackground(new Color(0, 176, 59));
        setForeground(new Color(0, 176, 59));
        
        addMouseMotionListener(new MouseMotionAdapter(){
            public void mouseMoved(MouseEvent evt){
                SpeelVeldPanel.this.mouseMoved(evt);
            }
        });
	    
	    GridBagLayout gbl = new GridBagLayout();
	    setLayout(gbl);
	    gbc = new GridBagConstraints();
	    gbc.weightx = 1;
	    gbc.weighty = 1;

        //CREATEPANELS
        createMenuPanel();
		createSingleCellPanels(aantalFreeCells, aantalHomeCells,
                                    aantalStackCells, stackCellKaarten);
        createButtonPanel();

		//CREATE DIALOGS

		about = new AboutDialog("About", 220, 150, false);
		about.setVisible(false);

		help = new HelpDialog("Help", 450, 450, false);
		help.setVisible(false);

		kolom = new KolomDialog("Movement", 180, 100, false);
		help.setVisible(false);		
		kolom.addPropertyChangeListener(this);
	}
    
    //METHOD createSingleCellPanels
    public void createSingleCellPanels(int aantalFreeCells, int aantalHomeCells,
                                    int aantalStackCells, Vector[] stackCellKaarten){

        int aantalCells = aantalFreeCells + aantalHomeCells + aantalStackCells;
        singleCellPanels = new SingleCellPanel[aantalCells];

        //FACTORY AANMAKEN
        CellPanelFactory factory = new CellPanelFactory(this, singleCellPanels,
                        aantalFreeCells, aantalHomeCells, aantalStackCells, stackCellKaarten);

        //OPBOUWEN SPEELVELDPANEL AAN DE HAND VAN DE GEKOZEN FACTORY    
        createSpeelVeldPanel(factory);
	    
        //ACHTERGRONDEN VAN DE CELLS ZETTEN
        singleCellPanels[8].setBackgroundImage(GameFrame.backgrounds[0]);   //F
        singleCellPanels[9].setBackgroundImage(GameFrame.backgrounds[1]);   //R
        singleCellPanels[10].setBackgroundImage(GameFrame.backgrounds[2]);  //E
        singleCellPanels[11].setBackgroundImage(GameFrame.backgrounds[2]);  //E
        singleCellPanels[12].setBackgroundImage(GameFrame.backgrounds[3]);  //C
        singleCellPanels[13].setBackgroundImage(GameFrame.backgrounds[2]);  //E
        singleCellPanels[14].setBackgroundImage(GameFrame.backgrounds[4]);  //L
        singleCellPanels[15].setBackgroundImage(GameFrame.backgrounds[4]);  //L
    }

    //METHOD createSpeelVeldPanel
    public void createSpeelVeldPanel(CellPanelFactory factory){

        LightWeightPanel buttonPanel = factory.createFreeCellPanel();
        
		LightWeightPanel freeCellPanel = factory.createFreeCellPanel();
        gbc.insets = new Insets(5, 4, 3, 5);
        gbc.anchor = GridBagConstraints.NORTHWEST;
        add(this, freeCellPanel, gbc, 0, 1, 1, 1);
        
        headPanel = factory.createHeadPanel();
        gbc.anchor = GridBagConstraints.NORTH;
        add(this, headPanel, gbc, 1, 1, 1, 1);
        
        LightWeightPanel homeCellPanel = factory.createHomeCellPanel();
        gbc.anchor = GridBagConstraints.NORTHEAST;
        add(this, homeCellPanel, gbc, 2, 1, 1, 1);

        LightWeightPanel stackCellPanel = factory.createStackCellPanel();
        gbc.weighty = 100;
        gbc.anchor = GridBagConstraints.NORTH;
        add(this, stackCellPanel, gbc, 0, 3, 3, 1);
    }

    //METHOD createButtonPanel
    public void createButtonPanel(){
    	buttonPanel = new Panel();
    	buttonPanel.setForeground(Color.black);
    	
    	//ADD THE BUTTONS
    	addButton(buttonPanel, "Ace");
    	for (int i = 2; i <= 10; i++)
    		addButton(buttonPanel, "" +i +"");
    	addButton(buttonPanel, "Jack");
    	addButton(buttonPanel, "Queen");
    	addButton(buttonPanel, "King");
    	
    	gbc.weightx = 100;
	    gbc.weighty = 0;
	    gbc.anchor = GridBagConstraints.SOUTH;
	    gbc.fill = GridBagConstraints.HORIZONTAL;
    	add(this, buttonPanel, gbc, 0, 4, 3, 1);
    }
    
    public void createMenuPanel(){
    	menuPanel = new Panel();
    	menuPanel.setForeground(Color.black);
    	
    	//ADD THE BUTTONS
    	addButton(menuPanel, "New Game");
    	addButton(menuPanel, "Restart Game");
    	addButton(menuPanel, "Help");
    	addButton(menuPanel, "About");
        textField = new TextField("  Welcome to Freecell !  ");
        textField.setEditable(false);
        textField.setBackground(Color.white);
        textField.setForeground(Color.black);
	    menuPanel.add(textField);
    	
    	gbc.weightx = 100;
	    gbc.weighty = 0;
	    gbc.anchor = GridBagConstraints.SOUTH;
	    gbc.fill = GridBagConstraints.HORIZONTAL;
    	add(this, menuPanel, gbc, 0, 0, 3, 1);
    }

    //METHOD addButton
    private void addButton(Container container, String string){
    	Button button = new Button(string);
    	container.add(button);
    	button.addActionListener(this);
    }
    
    //METHOD actionPerformed
    public void actionPerformed(ActionEvent evt){
    	String command = evt.getActionCommand();
    	if (command.equals("New Game"))
		{
			pcs.firePropertyChange("New Game",new Boolean(true),null);		
		}
		else if (command.equals("Restart Game"))
		{
			pcs.firePropertyChange("Restart Game",new Boolean(true),null);				
		}
		else if (command.equals("About"))
		{
			about.setVisible(true);
		}
		else if (command.equals("Help"))
		{
			help.setVisible(true);
		}
		else
		{
			byte b;
			if (command.equals("Ace")) b = 1;
    		else if (command.equals("Jack")) b = 11;
    		else if (command.equals("Queen")) b = 12;
    		else if (command.equals("King")) b = 13;
    		else
			{
    			b = (Integer.valueOf(command)).byteValue();
    		}
			pcs.firePropertyChange("Where",new Byte(b),null);
		}
	}
    
    //METHOD add 
    public void add(Panel p, Component c, GridBagConstraints gbc, int x, int y, int w, int h) {
    	gbc.gridx = x;
	    gbc.gridy = y;
	    gbc.gridwidth = w;
	    gbc.gridheight = h;
	    p.add(c, gbc);
    }

    //METHOD propertyChange
    public void propertyChange(PropertyChangeEvent evt){
        String property = evt.getPropertyName();
        if (property.equals("clicked")){
            if (!selected) {
            	fromId = ((Integer)evt.getNewValue()).intValue();
                selected = singleCellPanels[fromId].select();
            }
            else 
			{
                toId = ((Integer)evt.getNewValue()).intValue();
                selected = singleCellPanels[fromId].deselect();
            	try{
            	    vcs.fireVetoableChange("change",new Integer(fromId), new Integer(toId));
                    pcs.firePropertyChange("Model", new Integer(fromId), new Integer(toId));
            	}
            	catch(PropertyVetoException e){}
            }
        }
        else if (property.equals("veranderd")){
            int from = ((ObjectContainer)evt.getOldValue()).getIndex();
            int to = ((ObjectContainer)evt.getNewValue()).getIndex();
            singleCellPanels[from].drawCell(((ObjectContainer)evt.getOldValue()).getObject());
            singleCellPanels[to].drawCell(((ObjectContainer)evt.getNewValue()).getObject());
        }
        else if (property.equals("newCardDeck")){
            int cellNr = ((ObjectContainer)evt.getOldValue()).getIndex();
            selected = false;
            singleCellPanels[cellNr].deselect();
            singleCellPanels[cellNr].drawCell(((ObjectContainer)evt.getOldValue()).getObject());
        }
        else if (property.equals("rightMouseClick")){
            String kaartNaam = (String) evt.getOldValue();
            textField.setText(kaartNaam);
        }
        else if (property.equals("TextField")){
            String s = (String)evt.getOldValue();
            textField.setText(s);
        }
        else if (property.equals("Victory")){
            EndDemo demo = new EndDemo (singleCellPanels);
            demo.start();
        }
        else if (property.equals("Kupke")){
        	int id = ((Integer)evt.getOldValue()).intValue();
        	if (id <= 3 || (id <= 11 && id >= 8)){
        		if (!headPanel.left)
                	headPanel.setHeadToLeft(true);
        	}
        	else if (headPanel.left)
        		headPanel.setHeadToLeft(false);
        }
        else if (property.equals("Flash")){
            Vector v = (Vector)evt.getOldValue();
            byte rang = ((Byte)evt.getNewValue()).byteValue();
            Enumeration enum = v.elements();
            while (enum.hasMoreElements()){
                byte panel = ((Byte)enum.nextElement()).byteValue();
                singleCellPanels[panel].highlightCard(rang);
            }
            try{
                    Thread.sleep(1000);
            }
            catch(InterruptedException e){}
            enum = v.elements();
            while (enum.hasMoreElements()){
                byte panel = ((Byte)enum.nextElement()).byteValue();
                singleCellPanels[panel].highlightCard(rang);
            }
        }
		else if (property.equals("KolomDialog"))
		{
			kolom.setVisible(true);
		}
		else if (property.equals("DIALOG_KOLOM"))
		{
			pcs.firePropertyChange("VERPLAATS_KOLOM", new Integer(fromId), new Integer(toId));
		}
		else if (property.equals("DIALOG_KAART"))
		{
			pcs.firePropertyChange("VERPLAATS_KAART", new Integer(fromId), new Integer(toId));
		}
     }
    
    //METHOD mouseMoved
    public void mouseMoved(MouseEvent evt){
        int panelWidth = getSize().width;
        int x = evt.getX();
        if (x < panelWidth/2){
            if (!headPanel.left)
                headPanel.setHeadToLeft(true);
        }
        else if (headPanel.left)
            headPanel.setHeadToLeft(false);
    }
    
    //METHOD addPropertyChangeListener(
    public void addPropertyChangeListener(PropertyChangeListener l){
	    pcs.addPropertyChangeListener(l);
    }
	
    //METHOD removePropertyChangeListener
    public void removePropertyChangeListener(PropertyChangeListener l){
	    pcs.removePropertyChangeListener(l);
    }

    //METHOD addVetoableChangeListener(
    public void addVetoableChangeListener(VetoableChangeListener l){
        vcs.addVetoableChangeListener(l);
	}
	
    //METHOD removeVetoableChangeListener
    public void removeVetoableChangeListener(VetoableChangeListener l){
        vcs.removeVetoableChangeListener(l);
	}
}
