/* Copyright (C) 2001 W.P. van Paassen - peter@paassen.tmfweb.nl, E.C. Brummel
   

   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

//***************************************************************************
//***                       CLASS SPEL                                    ***
//***************************************************************************

import java.beans.*;
import java.util.*;

class Spel implements PropertyChangeListener, VetoableChangeListener {

    public static short[][] zetten;

    private SpeelVeld speelveld;
    private int aantalFreeCells;
    private int aantalHomeCells;
    private int aantalStackCells;
    private int aantalKaarten;
    private Kaart[] kaartenRij;
    private PropertyChangeSupport pcs = new PropertyChangeSupport(this);
    private boolean aanHetUitleggen = false;
    private int sleepTime = 500;
    private boolean tweedeVeto = false;
	private Vector kolom;
    private int toId;
	private int fromId;
	
    //CONSTRUCTOR
    public Spel(Kaart[] kaartenRij, int aantalFreeCells, int aantalHomeCells, int maxAantalKolommen){

        this.aantalFreeCells = aantalFreeCells;
    	this.aantalHomeCells = aantalHomeCells;
        this.aantalKaarten = kaartenRij.length;
        this.kaartenRij = kaartenRij;
        if (aantalKaarten < maxAantalKolommen) aantalStackCells = aantalKaarten;
        else aantalStackCells = maxAantalKolommen;

    	//ALLE MOGELIJKE ZETTEN MAKEN
    	maakAlleMogelijkeZetten();
        
        //SPEELVELD OPBOUWEN
        speelveld = new SpeelVeld(kaartenRij, aantalFreeCells, aantalHomeCells, aantalStackCells);
    }
    
    //=================================================================================
    //                              METHODEN VOOR GUI
    //=================================================================================
    
    //METHOD getAantalFreeCells
    public int getAantalFreeCells(){ return aantalFreeCells; }
    
    //METHOD getAantalHomeCells
    public int getAantalHomeCells(){ return aantalHomeCells; }
    
    //METHOD getAantalStackCells
    public int getAantalStackCells(){ return aantalStackCells; }
 
    //METHOD getKaartenUitStackCell
    public Vector getKaartenUitStackCell(int cellNr){
        return speelveld.getKaartenUitStackCell(cellNr);
    }

    //METHOD getAantalKaarten
    public int getAantalKaarten(){
        return aantalKaarten;
    }
    
    //METHOD getAantalLegeFreeCells
    public int getAantalLegeFreeCells(){
    	return speelveld.getAantalLegeFreeCells();
    }
    
    //METHOD getAantalLegeStackCells
    public int getAantalLegeStackCells(){
    	return speelveld.getAantalLegeStackCells();
    }

    //METHOD createNewCardDeck
    public void createNewCardDeck(int aantalKaarten){
        //NIEUWE KAARTSTAPEL MAKEN EN AAN MODEL GEVEN

        KaartStapel kaartStapel = new KaartStapel(aantalKaarten);
        kaartenRij = kaartStapel.getKaartenRij();
        speelveld.createNewCardDeck(kaartStapel.getKaartenRij());
        
        //GUI OP DE HOOGTE STELLEN VAN VERANDERING
        int aantalCells = aantalHomeCells + aantalStackCells + aantalFreeCells;
        for (int i = 0; i < aantalCells; i++){
            ObjectContainer oc = new ObjectContainer(speelveld.geefCellInhoud(i),i);
            pcs.firePropertyChange("newCardDeck", oc, null);
            pcs.firePropertyChange("TextField",new String(""),null);
        }
    }
    
    //METHOD createSameCardDeck
    public void createSameCardDeck(){

        //NIEUWE KAARTSTAPEL MAKEN EN AAN MODEL GEVEN
        speelveld.createNewCardDeck(kaartenRij);
        
        //GUI OP DE HOOGTE STELLEN VAN VERANDERING
        int aantalCells = aantalHomeCells + aantalStackCells + aantalFreeCells;
        for (int i = 0; i < aantalCells; i++){
            ObjectContainer oc = new ObjectContainer(speelveld.geefCellInhoud(i),i);
            pcs.firePropertyChange("newCardDeck", oc, null);
            pcs.firePropertyChange("TextField",new String(""),null);
        }
    }
    
    //METHOD setSleepTime
    public void setSleepTime(int sleepTime){
    	this.sleepTime = sleepTime;
    }
    //METHODEN VOOR KOLOMVERPLAATSER
    
    //METHOD geefBeginKolom
    public Vector geefBeginKolom(Cell fromCell, Cell toCell, int fromId, int toId){
        if (fromCell.getAantalKaarten() < 2 || !(toCell instanceof Stack_Cell)) return null;
        
        Cell kopie = fromCell.clonedeep();
        Vector kaarten = ((Stack_Cell)fromCell).getKaarten();
        Vector containers = new Vector();
        int[] vrijeFreeCellPlaatsen = geefVrijePlaatsenFreeCell();
        int[] vrijeStackCellPlaatsen = geefVrijePlaatsenStackCell();
        int aantalVrijePlaatsen = getAantalLegeFreeCells() + getAantalLegeStackCells(); 
        int i  = 0;
     
        while (aantalVrijePlaatsen >= 0 && i < fromCell.getAantalKaarten()-1){
                
            //is er wel een kolom
            Kaart kaart = kopie.getTopKaart();
            kopie.removeTopKaart();
            if (!kopie.kaartKanGeplaatstWorden(kaart) || (toCell.kaartKanGeplaatstWorden(kaart) && toCell.bevatKaart()))
                 break;
            else{
                //waar kan de kaart geplaatst worden, zijn er nog genoeg beschikbare plaatsen
                boolean kaartInStackCellGeplaatst = false;
                    
                //eerst kijken of de kaart niet op een (volle) stackcell kan
                   
                for (int j = 0; j < aantalStackCells; j++){
                    Cell stack_Cell = speelveld.getCell(j);
                    if (stack_Cell.bevatKaart() && stack_Cell.kaartKanGeplaatstWorden((Kaart)kaarten.elementAt(i)) &&  j != fromId && j != toId){
                        ObjectContainer zet = new ObjectContainer(kaarten.elementAt(i),j);
                        containers.addElement(zet);
                        j = aantalStackCells; //nu wordt de eerste for-loop beeindigd
                        kaartInStackCellGeplaatst = true;
                     }
                }
                if (aantalVrijePlaatsen == 0 && !kaartInStackCellGeplaatst) 
					break;
                if (aantalVrijePlaatsen > 0){
                        //geen plek op een volle stackcell dan de lege stackcells proberen
                    
                        if (!kaartInStackCellGeplaatst){
                            for (int j = 0; j < aantalStackCells; j++){
                                Cell stack_Cell = speelveld.getCell(j);
                                if (!cellAlBezet(j, vrijeStackCellPlaatsen) && !stack_Cell.bevatKaart() &&  j != fromId && j != toId){
                                    ObjectContainer zet = new ObjectContainer(kaarten.elementAt(i),j);
                                    containers.addElement(zet);
                                    j = aantalStackCells; //nu wordt de tweede for-loop beeindigd
                                    kaartInStackCellGeplaatst = true;
                                    aantalVrijePlaatsen--;
                                } 
                            }
                        }       
                        //geen plek in de stackcells dan een freecell proberen

                        if (!kaartInStackCellGeplaatst){
                            aantalVrijePlaatsen--;
                            for (int k = 0; k < getAantalLegeFreeCells(); k++){
                                if (vrijeFreeCellPlaatsen[k] != -1){
                                    ObjectContainer zet = new ObjectContainer(kaarten.elementAt(i),vrijeFreeCellPlaatsen[k]);
                                    containers.addElement(zet);
                                    vrijeFreeCellPlaatsen[k] = -1;
                                    k = getAantalLegeFreeCells(); //loop beeindigen
                                }
                            }
                        }
                    }
                    i++;    
            }
        }
        //laatste kaart ook meenemen
                    
        ObjectContainer zet = new ObjectContainer(kaarten.elementAt(i),toId);
        containers.addElement(zet);
        
        if (toCell.bevatKaart() && !toCell.kaartKanGeplaatstWorden((Kaart)(kaarten.elementAt(i)))) 
			return null;
        
        return containers;
    }
    
    //METHOD geefVrijePlaatsenFreeCell
    public int[] geefVrijePlaatsenFreeCell(){
        int aantal = getAantalLegeFreeCells();
        int[] vrij = new int[aantal];
        
        //rij vullen met lege plaatsen
        
        for (int i = 8, j = 0; i < 12; i++){
            Cell cell = speelveld.getCell(i);
            if (!cell.bevatKaart()){
                vrij[j] = i;
                j++;
            }
        }
        return vrij;
    }
    
     //METHOD geefVrijePlaatsenStackCell
    public int[] geefVrijePlaatsenStackCell(){
        int aantal = getAantalLegeStackCells();
        int[] vrij = new int[aantal];
        
        //rij vullen met lege plaatsen
        
        for (int i = 0, j = 0; i < 8; i++){
            Cell cell = speelveld.getCell(i);
            if (!cell.bevatKaart()){
                vrij[j] = i;
                j++;
            }
        }
        return vrij;
    }
    
    //METHOD cellAlBezet
    public boolean cellAlBezet(int index, int[] vrijeStackCellPlaatsen){
        for (int i = 0; i < getAantalLegeStackCells(); i++){
            if (vrijeStackCellPlaatsen[i] == index){
                vrijeStackCellPlaatsen[i] = -1;
                return false;
            }
        }
        return true;
    }
    
    //METHOD vetoableChange
    public void vetoableChange(PropertyChangeEvent evt) throws PropertyVetoException{
        if (tweedeVeto){
            tweedeVeto = false;
            return;
        }
        String property = evt.getPropertyName();
        if (property.equals("change")){		
            fromId = ((Integer)evt.getOldValue()).intValue();
            toId = ((Integer)evt.getNewValue()).intValue();

            Cell fromCell = speelveld.getCell(fromId);
            Cell toCell = speelveld.getCell(toId);
          
            kolom = geefBeginKolom(fromCell, toCell, fromId, toId);
			if (!toCell.kaartKanGeplaatstWorden(fromCell.getTopKaart()) ||
                (fromCell instanceof Stack_Cell && toCell instanceof Stack_Cell &&
                    kolom != null))
			{   
				if (kolom == null)
				{
				    tweedeVeto = true;
				    pcs.firePropertyChange("TextField",new String("Illegal move!"),null);
				    throw new PropertyVetoException("Kaart kan niet geplaatst worden", evt);
				}
				if (!toCell.bevatKaart() && kolom.size() > 1)
				{
					tweedeVeto = true;
				    pcs.firePropertyChange("KolomDialog", new Boolean(true), null);
					throw new PropertyVetoException("Kaart kan niet geplaatst worden", evt);
				}

				tweedeVeto = true;
				Verplaatsen();
				throw new PropertyVetoException("Kaart kan niet geplaatst worden", evt);
			}
			//oke no veto!!
        }
    }
	
    public void Verplaatsen()
	{
		for (int i = 0; i < kolom.size(); i++)
		{
			ObjectContainer container = (ObjectContainer)kolom.elementAt(i);
	        changeModel(fromId, container.getIndex());
	    }
		                    
	    //verplaatste kaarten naar lege posities ook weer terugzetten 
	    //let op de card collector kan ze hebben weggehaald!!!!
		                    
	    for (int i = kolom.size() - 2; i >= 0; i--)
		{
			ObjectContainer container = (ObjectContainer)kolom.elementAt(i);
	        if (speelveld.getCell(container.getIndex()).bevatKaart())
			{
				//geen kaart terugzetten die door de cardcollector is verwijderd!
	            if ((speelveld.getCell(container.getIndex())).getTopKaart().equals((Kaart)container.getObject()))
					changeModel(container.getIndex(), toId);
	        }
	    }
	}

	private void changeModel(int fromId, int toId)
	{
		speelveld.changeZetIndex(fromId,toId);
        speelveld.changeState();
		
        if (speelveld.inFinalState()){
            pcs.firePropertyChange("TextField",new String("Congratulations!"),null);
            pcs.firePropertyChange("Victory",new Boolean(true),null);
        }
		
		ObjectContainer oc1 = new ObjectContainer(speelveld.geefCellInhoud(fromId),fromId);
		ObjectContainer oc2 = new ObjectContainer(speelveld.geefCellInhoud(toId),toId);
		pcs.firePropertyChange("veranderd",oc1,oc2);
		
		//OVERBODIGE KAARTEN OP DE HOMECELLS LEGGEN
		int[] fromToRij = speelveld.geefOverbodigeKaart();
		if (fromToRij != null)
		{
			changeModel(fromToRij[0], fromToRij[1]);
		}
	}
    
    //METHOD propertyChange
    public void propertyChange(PropertyChangeEvent evt){
        String property = evt.getPropertyName();
        
        if (property.equals("Model"))
		{
            int fromId = ((Integer)evt.getOldValue()).intValue();
            int toId = ((Integer)evt.getNewValue()).intValue();
			changeModel(fromId, toId);
		}
        if (property.equals("Where"))
		{
            byte rang = ((Byte)evt.getOldValue()).byteValue();
            Vector locaties = speelveld.geefLocatiesKaart(rang);
            pcs.firePropertyChange("Flash",locaties,new Byte(rang));
        }
		else if (property.equals("New Game"))
		{
			createNewCardDeck(52);			
		}
		else if (property.equals("Restart Game"))
		{
			createSameCardDeck();
		}
		else if (property.equals("VERPLAATS_KOLOM"))
		{
			Verplaatsen();
		}
		else if (property.equals("VERPLAATS_KAART"))
		{
            fromId = ((Integer)evt.getOldValue()).intValue();
            toId = ((Integer)evt.getNewValue()).intValue();
			changeModel(fromId, toId);
		}		
    }
    
    //METHOD addPropertyChangeListener(
	public void addPropertyChangeListener(PropertyChangeListener l){
		pcs.addPropertyChangeListener(l);
	}
	
	//METHOD removePropertyChangeListener
	public void removePropertyChangeListener(PropertyChangeListener l){
		pcs.removePropertyChangeListener(l);
	}
	
    //METHOD maakAlleMogelijkeZetten
    public void maakAlleMogelijkeZetten(){
        zetten = new short[228][2];
        //VUL ALLE MOGELIJKE ZETTEN IN DE ZETTEN-ARRAY IN
       	short from = 0;
       	short to = 12;
   	    for (int index = 1; index <= 48; index++){
       		zetten[index-1][0] = from;
      		zetten[index-1][1] = to;
                from++;
                if (index % 12 == 0){
       			from = 0;
       			to++;
       		}
      	}
      	from = 1;
       	to = 0;
        for (int index = 1; index <= 180; index++){
       		zetten[48 + index-1][0] = from;
       		zetten[48 + index-1][1] = to;
                from++;
       		if (from == to) from++;
                if (from == 16){
                        from = 0;
                        to++;
                }
      	}
    }
}
